From 991bf4f7c3d58cb8d8858cf4284c360719843350 Mon Sep 17 00:00:00 2001
From: "Matthew \"strager\" Glazar" <strager.nds@gmail.com>
Date: Fri, 27 Jan 2023 16:19:45 -0800
Subject: [PATCH] gdb/tui: add 'set tui mouse-events off' to restore mouse
 selection

Rationale:
I use the mouse with my terminal to select and copy text. In gdb, I use
the mouse to select a function name to set a breakpoint, or a variable
name to print, for example.

When gdb is compiled with ncurses mouse support, gdb's TUI mode
intercepts mouse events. Left-clicking and dragging, which would
normally select text, seems to do nothing. This means I cannot select
text using my mouse anymore. This makes it harder to set breakpoints,
print variables, etc.

Solution:
I tried to fix this issue by editing the 'mousemask' call to only enable
buttons 4 and 5. However, this still caused my terminal (gnome-terminal)
to not allow text to be selected. The only way I could make it work is
by calling 'mousemask (0, NULL);'. But doing so disables the mouse code
entirely, which other people might want.

I therefore decided to make a setting in gdb called 'tui mouse-events'.
If enabled (the default), the behavior is as it is now: terminal mouse
events are given to gdb, disabling the terminal's default behavior.
If disabled (opt-in), the behavior is as it was before the year 2020:
terminal mouse events are not given to gdb, therefore the mouse can be
used to select and copy text.

Notes:
I am not attached to the setting name or its description. Feel free to
suggest better wording.

Testing:
I tested this change in gnome-terminal by performing the following steps
manually:

1. Run: gdb --args ./myprogram
2. Enable TUI: press ctrl-x ctrl-a
3. Click and drag text with the mouse. Observe no selection.
4. Input: set tui mouse-events off
5. Click and drag text with the mouse. Observe that selection works now.
6. Input: set tui mouse-events on
7. Click and drag text with the mouse Observe no selection.
---
 gdb/NEWS            |  7 +++++++
 gdb/doc/gdb.texinfo | 13 +++++++++++++
 gdb/tui/tui-io.c    |  3 ++-
 gdb/tui/tui-win.c   | 24 ++++++++++++++++++++++++
 gdb/tui/tui-win.h   |  3 +++
 5 files changed, 49 insertions(+), 1 deletion(-)

diff --git gdb/NEWS gdb/NEWS
index 2bc1672632a..c14158e41b5 100644
--- gdb/NEWS
+++ gdb/NEWS
@@ -17,6 +17,13 @@ maintenance print record-instruction [ N ]
   prints how GDB would undo the N-th previous instruction, and if N is
   positive, it prints how GDB will redo the N-th following instruction.
 
+set tui mouse-events [on|off]
+show tui mouse-events
+  When on (default), the TUI will interpret mouse clicks (including scroll wheel
+  presses), overwriting the terminal's default behavior (usually text
+  selection). When off, the TUI will preserve the terminal's default behavior
+  for mouse events.
+
 * MI changes
 
 ** mi now reports 'no-history' as a stop reason when hitting the end of the
diff --git gdb/doc/gdb.texinfo gdb/doc/gdb.texinfo
index b5fad2cb16e..c36c24273d3 100644
--- gdb/doc/gdb.texinfo
+++ gdb/doc/gdb.texinfo
@@ -29867,6 +29867,19 @@ source text at the next tab stop; the compact display uses only as
 much space as is needed for the line numbers in the current file, and
 only a single space to separate the line numbers from the source.
 
+@item set tui mouse-events @r{[}on@r{|}off@r{]}
+@kindex set tui mouse-events
+When on (default), the TUI will interpret mouse clicks (including scroll wheel
+presses), overwriting the terminal's default behavior (usually text selection).
+When off, the TUI will preserve the terminal's default behavior for mouse
+events.
+
+Set whether the TUI source window is displayed in ``compact'' form.
+The default display uses more space for line numbers and starts the
+source text at the next tab stop; the compact display uses only as
+much space as is needed for the line numbers in the current file, and
+only a single space to separate the line numbers from the source.
+
 @kindex set debug tui
 @item set debug tui @r{[}on|off@r{]}
 Turn on or off display of @value{GDBN} internal debug messages relating
diff --git gdb/tui/tui-io.c gdb/tui/tui-io.c
index a17cf38a1f2..45ef34aeab5 100644
--- gdb/tui/tui-io.c
+++ gdb/tui/tui-io.c
@@ -655,7 +655,8 @@ static void
 tui_prep_terminal (int notused1)
 {
 #ifdef NCURSES_MOUSE_VERSION
-  mousemask (ALL_MOUSE_EVENTS, NULL);
+  if (tui_enable_mouse)
+    mousemask (ALL_MOUSE_EVENTS, NULL);
 #endif
 }
 
diff --git gdb/tui/tui-win.c gdb/tui/tui-win.c
index 008189eb99b..5709bfcbac1 100644
--- gdb/tui/tui-win.c
+++ gdb/tui/tui-win.c
@@ -897,6 +897,17 @@ tui_show_compact_source (struct ui_file *file, int from_tty,
   gdb_printf (file, _("TUI source window compactness is %s.\n"), value);
 }
 
+bool tui_enable_mouse = true;
+
+/* Implement 'show tui mouse-events'.  */
+
+static void
+show_tui_mouse_events (struct ui_file *file, int from_tty,
+		       struct cmd_list_element *c, const char *value)
+{
+  gdb_printf (file, _("TUI mouse events is %s.\n"), value);
+}
+
 /* Set the tab width of the specified window.  */
 static void
 tui_set_tab_width_command (const char *arg, int from_tty)
@@ -1284,6 +1295,19 @@ position indicator is styled."),
 			   &style_set_list,
 			   &style_show_list);
 
+  add_setshow_boolean_cmd ("mouse-events", class_tui,
+			   &tui_enable_mouse, _("\
+Set whether TUI mode handles mouse clicks."), _("\
+Show whether TUI mode handles mouse clicks."), _("\
+When on, the TUI will interpret mouse clicks (including scroll wheel presses),\n\
+overwriting the terminal's default behavior (usually text selection).\n\
+When off, the TUI will preserve the terminal's default behavior for mouse\n\
+events."),
+
+			   NULL,
+			   show_tui_mouse_events,
+			   &tui_setlist, &tui_showlist);
+
   tui_border_style.changed.attach (tui_rehighlight_all, "tui-win");
   tui_active_border_style.changed.attach (tui_rehighlight_all, "tui-win");
 }
diff --git gdb/tui/tui-win.h gdb/tui/tui-win.h
index 4b33f1f2b54..039993fba34 100644
--- gdb/tui/tui-win.h
+++ gdb/tui/tui-win.h
@@ -51,6 +51,9 @@ struct cmd_list_element **tui_get_cmd_list (void);
 /* Whether compact source display should be used.  */
 extern bool compact_source;
 
+/* Whether the TUI should intercept terminal mouse events.  */
+extern bool tui_enable_mouse;
+
 /* Whether to style the source and assembly code highlighted by the TUI's
    current position indicator.  */
 extern bool style_tui_current_position;
-- 
2.29.2

